/*
    $Id: SCContextMenuView.java,v 1.3 2005/04/28 19:10:28 phonohawk Exp $
    ----------
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2005 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

public class SCContextMenuView extends NSMenuView {
    //private static boolean isTransparent = true;
    //private static float transparency = 0.5f; // sɂꍇ1.0fw肵ĉB
    
    SCSession session;
    boolean hasSideBar;
    int scope;
    
    SCMenuAppearanceServer mas;
    int widthOfSideBar;
    
    public SCContextMenuView(SCSession session, int scope, boolean hasSideBar) {
	super();
	this.session = session;
	this.scope = scope;
	this.hasSideBar = hasSideBar;
	
	mas = session.getMenuAppearance();
	if (mas.hasImage() && hasSideBar) {
	    widthOfSideBar = (int)mas.getSideImage(scope).size().width();
	}
	else {
	    this.hasSideBar = false;
	    widthOfSideBar = 0;
	}
    }
    
    public SCContextMenuView(SCSession session, int scope) {
	super();
	this.session = session;
	this.scope = scope;
	this.hasSideBar = false;
	
	mas = session.getMenuAppearance();
	widthOfSideBar = 0;
    }
    
    public boolean isOpaque() {
	return true;
    }
    
    public int getSidebarWidth() {
	return widthOfSideBar;
    }
    
    public float stateImageOffset() {
	return (hasSideBar ? 0 : super.stateImageOffset());
    }
    
    public float imageAndTitleOffset() {
	return (hasSideBar ? 5 : super.imageAndTitleOffset());
    }

    public void drawRect(NSRect r) {
	if (!mas.hasImage()) { // ftHg̃j[
	    super.drawRect(r);
	    return;
	}
		
	// TCho[
	if (hasSideBar) {
	    NSImage image = mas.getSideImage(scope);
	    NSColor topLeftColor = mas.getTopLeftColorOfSideImage(scope);
	    NSColor bottomRightColor = mas.getBottomRightColorOfSideImage(scope);
	    
	    float heightOfSideBar = image.size().height();
	    switch (mas.getAlignmentOfSide()) {
	    case SCMenuAppearanceServer.ALIGN_TOP: {
		float height = (heightOfSideBar > frame().height() ? frame().height() : heightOfSideBar);
		float srcY = (frame().height() > heightOfSideBar ? 0 : heightOfSideBar - frame().height());
		NSRect srcRect = new NSRect(0,srcY,widthOfSideBar,height);

		image.compositeToPointFromRect(
		    (isFlipped() ? new NSPoint(0,height) : new NSPoint(0,frame().height() - height)),
		    srcRect,
		    NSImage.CompositeCopy);

		if (frame().height() > heightOfSideBar) {
		    NSRect rectToFill = new NSRect(
			0,
			(isFlipped() ? heightOfSideBar : 0),
			widthOfSideBar,
			frame().height() - heightOfSideBar);
		    bottomRightColor.set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
		break;
		
	    case SCMenuAppearanceServer.ALIGN_BOTTOM: {
		float height = (heightOfSideBar > frame().height() ? frame().height() : heightOfSideBar);
		NSRect srcRect = new NSRect(0,0,widthOfSideBar,height);

		image.compositeToPointFromRect(
		    (isFlipped() ? new NSPoint(0,frame().height()) : NSPoint.ZeroPoint),
		    srcRect,
		    NSImage.CompositeCopy);
					
		if (frame().height() > heightOfSideBar) {
		    NSRect rectToFill = new NSRect(
			0,
			(isFlipped() ? 0 : heightOfSideBar),
			widthOfSideBar,
			frame().height() - heightOfSideBar);
		    topLeftColor.set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
		break;
	    }
	}
		
	// obNOEh
	float widthOfBg = mas.getBgImage().size().width();
	float heightOfBg = mas.getBgImage().size().height();
	float destwidth = frame().width()-widthOfSideBar;
	switch (mas.getAlignmentOfBg()) {
	case	SCMenuAppearanceServer.ALIGN_LEFTTOP: {
	    float width = (widthOfBg > destwidth ? destwidth : widthOfBg);
	    float height = (heightOfBg > frame().height() ? frame().height() : heightOfBg);
	    float y = (frame().height() > heightOfBg ? 0 : heightOfBg - frame().height());
	    NSRect srcRect = new NSRect(0,y,width,height);
	    NSPoint destPoint =
		(isFlipped() ?
		 new NSPoint(widthOfSideBar,height) :
		 new NSPoint(widthOfSideBar,frame().height() - height));

	    mas.getBgImage().compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				
	    // E̗]
	    if (destwidth > widthOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar+widthOfBg,
		    (isFlipped() ? 0 : frame().height() - height),
		    destwidth - widthOfBg,
		    height);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getBottomRightColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	    
	    // ̗]
	    if (frame().height() > heightOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? heightOfBg : frame().height() - heightOfBg),
		    destwidth,
		    frame().height() - heightOfBg);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getBottomRightColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	}
	    break;
	    
	case	SCMenuAppearanceServer.ALIGN_RIGHTTOP: {
	    float width = (widthOfBg > destwidth ? destwidth : widthOfBg);
	    float height = (heightOfBg > frame().height() ? frame().height() : heightOfBg);
	    float x = (destwidth > widthOfBg ? 0 : widthOfBg - destwidth);
	    float y = (frame().height() > heightOfBg ? 0 : heightOfBg - frame().height());
	    NSRect srcRect = new NSRect(x,y,width,height);
	    float destX = frame().width() - width;
	    NSPoint destPoint = (isFlipped() ? new NSPoint(destX,height) : new NSPoint(destX,frame().height() - height));
	    mas.getBgImage().compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
				
	    // ̗]
	    if (destwidth > widthOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? 0 : frame().height() - height),
		    destwidth - widthOfBg,
		    height);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getBottomRightColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
			
	    // ̗]
	    if (frame().height() > heightOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? heightOfBg : frame().height() - heightOfBg),
		    destwidth,
		    frame().height() - heightOfBg);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getBottomRightColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	}
	    break;
	    
	case	SCMenuAppearanceServer.ALIGN_LEFTBOTTOM: {
	    float width = (widthOfBg > destwidth ? destwidth : widthOfBg);
	    float height = (heightOfBg > frame().height() ? frame().height() : heightOfBg);
	    NSRect srcRect = new NSRect(0,0,width,height);
	    NSPoint destPoint = new NSPoint(widthOfSideBar,(isFlipped() ? frame().height() : 0));
	    mas.getBgImage().compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
	    
	    // E̗]
	    if (destwidth > widthOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar+widthOfBg,
		    (isFlipped() ? 0 : frame().height() - height),
		    destwidth - widthOfBg,
		    height);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getTopLeftColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	    
	    // ̗]
	    if (frame().height() > heightOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? 0 : heightOfBg),
		    destwidth,
		    frame().height() - heightOfBg);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getTopLeftColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	}
	    break;
	    
	case	SCMenuAppearanceServer.ALIGN_RIGHTBOTTOM: {
	    float width = (widthOfBg > destwidth ? destwidth : widthOfBg);
	    float height = (heightOfBg > frame().height() ? frame().height() : heightOfBg);
	    float srcX = (destwidth > widthOfBg ? 0 : widthOfBg - destwidth);
	    NSRect srcRect = new NSRect(srcX,0,width,height);
	    float destX = frame().width() - width;
	    NSPoint destPoint = new NSPoint(destX,(isFlipped() ? frame().height() : 0));
	    mas.getBgImage().compositeToPointFromRect(destPoint,srcRect,NSImage.CompositeCopy);
	    
	    // ̗]
	    if (destwidth > widthOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? 0 : frame().height() - height),
		    destwidth - widthOfBg,
		    height);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getTopLeftColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	    
	    // ̗]
	    if (frame().height() > heightOfBg) {
		NSRect region = new NSRect(
		    widthOfSideBar,
		    (isFlipped() ? 0 : heightOfBg),
		    destwidth,
		    frame().height() - heightOfBg);
		NSRect rectToFill = region.rectByIntersectingRect(r);
		if (rectToFill != NSRect.ZeroRect) {
		    mas.getTopLeftColorOfBgImage().set();
		    NSBezierPath.fillRect(rectToFill);
		}
	    }
	}
	    break;
	}
	
	// 
	final int n_items = menu().numberOfItems();
	for (int i = 0;i < n_items;i++) {
	    NSMenuItemCell mic = menuItemCellForItemAtIndex(i);
	    NSRect region = rectOfItemAtIndex(i);
	    
	    if (r.intersectsRect(region)) {
		mic.drawWithFrameInView(region,this);
	    }
	}
    }
    
    public static void printSuperRecursively(NSView view) {
	System.out.println(view.toString()+" : "+view.isFlipped());
	if (view.superview() != null) {
	    printSuperRecursively(view.superview());
	}
    }
}
