/*
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.util.*;

public class SCESSpecialEntry
{
    SCEseShiori eseshiori;
    
    String definition; // #EVNT=("OnBoot")("=00")
    String type; // EVNT , RESP , etc...
    Vector conditions; // content : (SCESMatchingSentence)}b`Op͒f
    Vector items; // content : (SCESItem)
    
    public SCESSpecialEntry(SCEseShiori eseshiori,String definition)
    {
	// definition ex. : #EVNT=("OnBoot")("=00")
	this.eseshiori = eseshiori;
	this.definition = definition;
	conditions = new Vector();
	items = new Vector();
	
	type = definition.substring(1,5);
	final int len = definition.length();
	boolean in_quoted = false;
	for (int pos = 0;pos < len;pos++)
	{
	    if (definition.charAt(pos) == '\"') in_quoted = !in_quoted;
	    if (definition.charAt(pos) != '(') continue;
	    
	    if (!in_quoted)
	    {
		int blockstart = pos;
		int blockend = definition.indexOf(')',pos);
		addCondition(definition.substring(blockstart+1,blockend));
	    }
	}
    }
    
    protected void addCondition(String cond)
    {
	/*
	    str [,startpos,endpos]
	*/
	SCESArgumentsTokenizer at = new SCESArgumentsTokenizer(cond,",");
	String str = eseshiori.getEvaluator().validateStringArgument(eseshiori.getEvaluator().evaluate(at.nextToken()));
	int startpos = -1,endpos = -1;
	if (at.hasMoreTokens())
	{
	    try
	    {
		startpos = Integer.parseInt(at.nextToken());
		endpos = Integer.parseInt(at.nextToken());
	    }
	    catch (Exception e)
	    {
		System.err.println("The EseShiori Clone : exception has occured in evaluating condition of special entries : "+cond);
	    }
	}
	conditions.add(new SCESMatchingSentence(str,startpos,endpos));
    }
    
    public String getDefinition()
    {
	return definition;
    }
    
    public void add(SCESItem item)
    {
	items.add(item);
    }
    
    public void add(int prob,String data)
    {
	items.add(new SCESItem(prob,data));
    }
    
    public boolean doMatchingWith(String str)
    {
	// str}b`邩ǂ𒲂ׂ
	if (conditions.size() == 0) return true; // Ζ}b`
	StringBuffer buf = new StringBuffer(str);
	for (Enumeration e = conditions.elements();e.hasMoreElements();)
	{
	    SCESMatchingSentence esms = (SCESMatchingSentence)e.nextElement();
	    if (esms.doMatchingWith(buf.toString()))
	    {
		// }b`Ă̂ō̕܂ł͏
		String matchingSentence = esms.getString();
		buf.delete(0,buf.toString().indexOf(matchingSentence)+matchingSentence.length());
	    }
	    else
	    {
		return false; // sv
	    }
	}
	return true; // SĈv
    }
    
    public String getNextItem()
    {
	// eڂ̊mIB
	// ܂͍vl߂
	final int n_items = items.size();
	if (n_items == 0) return null;
	
	int sum = 0;
	for (int i = 0;i < n_items;i++)
	{
	    sum += ((SCESItem)items.elementAt(i)).getProb();
	}
	
	final int randnum = (int)((Math.random() * 1000000) % sum);
	int floor = 0;
	for (int i = 0;i < n_items;i++)
	{
	    SCESItem item = (SCESItem)items.elementAt(i);
	    if (floor <= randnum && floor + item.getProb() > randnum)
	    {
		// Ɍ
		return item.getData();
	    }
	    else
	    {
		floor += item.getProb();
	    }
	}
	return null; // ɗ邱Ƃ͂肦ȂB
    }
}
