/*
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.io.*;
import java.util.*;

public class SCKawari
{
    public static final String KAWARI_CLONE_VERSION = "KAWARI.JAVA_CLONE/2.4";
    SCKawariDictionary dictionary;
    SCKawariEvaluator evaluator;
    SCKawariResponceDictionary respDict;
    File ghost_dir;
    
    private static Hashtable GET_WORD_CONVERT_TABLE;
    static
    {
	GET_WORD_CONVERT_TABLE = new Hashtable();
	GET_WORD_CONVERT_TABLE.put("\u00a5ms","compatible.ms");
	GET_WORD_CONVERT_TABLE.put("\u00a5mz","compatible.mz");
	GET_WORD_CONVERT_TABLE.put("\u00a5mc","compatible.mc");
	GET_WORD_CONVERT_TABLE.put("\u00a5mh","compatible.mh");
	GET_WORD_CONVERT_TABLE.put("\u00a5mt","compatible.mt");
	GET_WORD_CONVERT_TABLE.put("\u00a5me","compatible.me");
	GET_WORD_CONVERT_TABLE.put("\u00a5mp","compatible.mp");
	GET_WORD_CONVERT_TABLE.put("\u00a5m?","compatible.m");
	GET_WORD_CONVERT_TABLE.put("\u00a5dms","compatible.dms");
    }
    
    //----------- ȉ̏NOTIFYőĂ܂B------------//
    String ownerGhostName;
    //Vector otherGhostNames; // g͌܂łȂStringB
    //------------------------------------------------------------//
    
    public SCKawari(File ghost_dir)
    {
	this.ghost_dir = ghost_dir;
	dictionary = new SCKawariDictionary(this,ghost_dir);
	evaluator = new SCKawariEvaluator(this);
	respDict = new SCKawariResponceDictionary();
	
        if (!ghost_dir.exists() || !ghost_dir.isDirectory()) return;
        File[] items = ghost_dir.listFiles();
        
        for (int i = 0;i < items.length;i++)
        {
            if (items[i].getName().equals("kawari.ini"))
            {
                loadKawariIni(items[i]); // B
		break;
            }
        }
	
	// [hBCxgsB
	String sysOnLoad = dictionary.findEntryAndGetFirstElement("system.OnLoad");
	if (sysOnLoad != null) evaluator.evaluate(sysOnLoad); // ʂ͎̂ĂB
    }
    
    // private methods
    private void loadKawariIni(File inifile)
    {
        // ini[h܂B
        BufferedReader br = null;
        try
        {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(inifile),"SJIS"));
        }catch(Exception e){}
        
        String line = null;
        while(true)
        {
            // t@C̏I܂Ń[vB
            try{  line = br.readLine();  }catch(Exception e){}
            if (line == null) break; // EOF
	    if (line.length() == 0) continue; // s
	    if (line.charAt(0) == '#') continue; // comment
	    
	    int colonPos = line.indexOf(':');
	    if (colonPos == -1) continue; // RB
	    
	    String command = line.substring(0,colonPos).trim();
	    String data = line.substring(colonPos+1).trim();
            
            if (command.equals("dict"))
            {
                // t@C[hBÍĂĂĂȂĂǂB
		dictionary().loadDictFile(data);
            }
	    else if (command.equals("include"))
	    {
		// iniċAIɓǂݍށB
		// ݎQƂĂ玀ɂ܂B
		File target = new File(ghost_dir,SCKawariDictionary.convertWinPathToUnixOne(data));
		if (target.exists())
		{
		    loadKawariIni(target);
		}
	    }
	    else if (command.equals("adddict"))
	    {
		// ̃GgɒPǉB
		// pXy[Xŋ؂ĂB
		int space_pos = data.indexOf(' ');
		if (space_pos != -1)
		{
		    String entryname = data.substring(0,space_pos).trim();
		    String word = data.substring(space_pos+1).trim();
		    dictionary().addToken(entryname,word);
		}
	    }
	    else if (command.equals("set"))
	    {
		// ̃GgɒPZbgB
		// pXy[Xŋ؂ĂB
		int space_pos = data.indexOf(' ');
		if (space_pos != -1)
		{
		    String entryname = data.substring(0,space_pos).trim();
		    String word = data.substring(space_pos+1).trim();
		    dictionary().clearEntry(entryname);
		    dictionary().addToken(entryname,word);
		}
	    }
        }
	//dump();
	
	try{ br.close(); }catch(IOException e){}
    }
    
    // public methods
    public SCKawariDictionary dictionary()
    {
	return dictionary;
    }
    
    public SCKawariEvaluator evaluator()
    {
	return evaluator;
    }
    
    public SCKawariResponceDictionary getResponceDictionary()
    {
	return respDict;
    }
    
    public File getGhostRootDir()
    {
	return ghost_dir;
    }
        
    // implementations of the interface SCShioriKernel
    public static boolean checkIfUsable(File ghost_dir)
    {
        // ghost_dir̒kawari.iniTAȂfalseB
        /*if (!ghost_dir.exists() || !ghost_dir.isDirectory()) return false;
        File[] items = ghost_dir.listFiles();
        for (int i = 0;i < items.length;i++)
        {
            if (items[i].getName().equals("kawari.ini"))
            {
                return true;
            }
        }
        
        return false;*/
	return (new File(ghost_dir,"kawari.ini")).exists();
    }
    
    public void terminate()
    {
        // IBCxgsB
	String sysOnUnload = dictionary.findEntryAndGetFirstElement("system.OnUnload");
	if (sysOnUnload != null) evaluator.evaluate(sysOnUnload); // ʂ͎̂ĂB
    }
    
    public static String getModuleName()
    {
        //try{ return new String("ؘa".getBytes(),"SJIS"); }catch(Exception e){}
        //return null;
	return "\u83ef\u548c\u68a8";
    }
    
    public String getaistringrandom()
    {
        // AI_g[NłB
        // dictionary̒sentenceGgTA̒̈Ԃ܂B
	// ԂׂZeXnullԂ܂B
        Vector sentences = dictionary.findEntry("sentence");
        if (sentences == null)
        {
            return null;
        }
        int n_sentences = sentences.size();
        int id = (int)Math.round(Math.random() * (n_sentences-1));
        String result = (String)sentences.elementAt(id);
        
        result = evaluator.evaluateAndUnescape(result);
        
        return result;
    }
    
    public static Object selectObjectAtRandom(Vector vec)
    {
	// vecnullȂnullԂ܂B
	if (vec == null) return null;
	
	int select = (int)((Math.random() * Integer.MAX_VALUE) % vec.size());
	return vec.elementAt(select);
    }
    public static String selectStringAtRandom(Vector vec)
    {
	// vecnullȂnullԂ܂B
	return (String)selectObjectAtRandom(vec);
    }
    
    public String request(String req)
    {
	BufferedReader br = new BufferedReader(new StringReader(req));
	
	// ܂̓wb_ǂ
	String header = null;
	try{ header = br.readLine(); }catch (Exception e){}
	
	// NOTIFY OtherGhostName͑ĂGgdĂBN\I
	if (header.startsWith("NOTIFY OtherGhostName"))
	{
	    dictionary.clearEntry("system.OtherGhost");
	    dictionary.clearEntry("system.OtherGhostEx");
	    
	    String line = null;
	    while (true)
	    {
		try{ line = br.readLine(); }catch (Exception e){}
		if (line == null) break;
		
		if (line.startsWith("Ghost:"))
		{
		    dictionary.addToken("system.OtherGhost",line.substring(line.indexOf(':')+1).trim());
		}
		else if (line.startsWith("GhostEx:"))
		{
		    dictionary.addToken("system.OtherGhostEx",line.substring(line.indexOf(':')+1).trim());
		}
	    }
	    
	    try{ br.close(); }catch(IOException e){}
	    
	    // CxgsB
	    String sysONO = dictionary.findEntryAndGetFirstElement("system.OnNotifyOther");
	    if (sysONO != null) evaluator.evaluate(sysONO); // ʂ͎̂ĂB
	    
	    return "SHIORI/2.3 204 No Content\r\n\r\n";
	}
	
	// NGXgŜǂ
	Hashtable hash = new Hashtable();
	String line = null;
	while (true)
	{
	    try{ line = br.readLine(); }catch (Exception e){}
	    if (line == null) break;
	    if (line.indexOf(':') == -1) continue;
	    
	    String label = line.substring(0,line.indexOf(':'));
	    String data = line.substring(line.indexOf(':')+2); // R̎̃Xy[X͖
	    
	    hash.put(label,data);
	}
	
	// KvȃVXeGgZbg
	dictionary.clearEntry("system.Sender");
	if (hash.get("Sender") != null)
	{
	    dictionary.addToken("system.Sender",(String)hash.get("Sender"));
	}
	dictionary.clearEntry("system.Sender.Path");
	if (hash.get("SecurityLevel") != null)
	{
	    dictionary.addToken("system.Sender.Path",(String)hash.get("SecurityLevel"));
	}
	dictionary.clearEntry("system.Age");
	if (hash.get("Age") != null)
	{
	    dictionary.addToken("system.Age",(String)hash.get("Age"));
	}
	dictionary.clearEntry("system.Surface");
	if (hash.get("Surface") != null)
	{
	    dictionary.addToken("system.Surface",(String)hash.get("Surface"));
	}
	dictionary.clearEntry("system.Sentence");
	if (hash.get("Sentence") != null)
	{
	    dictionary.addToken("system.Sentence",(String)hash.get("Sentence"));
	}
	dictionary.clearEntry("system.EventName");
	if (hash.get("Event") != null)
	{
	    dictionary.addToken("system.EventName",(String)hash.get("Event"));
	}
	dictionary.clearEntry("system.communicate"); // ̃Gg͒gcĂƊԈɌĂт鋰ꂪB
	
	final int numOfReferencesToClear = 8; // system.Response0`7NABcĂƊ֌WȂRef𑗂Ă܂ꂪB
	for (int i = 0;i < numOfReferencesToClear;i++)
	{
	    dictionary.clearEntry("system.Response"+i);
	}
	
	
	// ߊJnB
	if (header.startsWith("GET Sentence"))
	{
	    if (hash.get("Event") != null) // Cxgt
	    {
		String eventName = (String)hash.get("Event");
		
		// Reference0 -> system.Reference0 o^
		final int numOfReferencesToSearch = 8;
		for (int i = 0;i < numOfReferencesToSearch;i++)
		{
		    String data = (String)hash.get("Reference"+i);
		    if (data == null) continue;
		    
		    dictionary.clearEntry("system.Reference"+i);
		    dictionary.addToken("system.Reference"+i,data);
		}
		
		// event.[name]BȂ204ԂB
		Vector evententry = null;
		evententry = dictionary.findEntry("event."+eventName);
		//if (evententry == null) evententry = dictionary.findEntry("event-"+eventName);
		//System.out.println(evententry == null ? 204 : 200);
		if (evententry == null)
		{
		    return "SHIORI/2.0 204 No Content\r\n\r\n";
		}
		else
		{
		    return generateResponce200(evaluator.evaluateAndUnescape(selectStringAtRandom(evententry)),hash);
		}
	    }
	    else if (hash.get("Sentence") != null) // ZeXt
	    {
		// Reference0 -> system.Reference0 o^
		final int numOfReferencesToSearch = 8;
		for (int i = 0;i < numOfReferencesToSearch;i++)
		{
		    String data = (String)hash.get("Reference"+i);
		    
		    dictionary.clearEntry("system.Reference"+i);
		    if (data != null) dictionary.addToken("system.Reference"+i,data);
		}
		
		Vector sysOnResp,hitResponces,respUnknownEntries;
		if ((sysOnResp = dictionary.findEntry("system.OnResponse")) != null) // system.OnResponse
		{
		    int n_sor = sysOnResp.size();
		    int id = (int)((Math.random() * 10000) % n_sor);
		    
		    return generateResponce200(
				evaluator.evaluateAndUnescape( (String)sysOnResp.elementAt(id) )
				,hash);
		}
		else if ((hitResponces = respDict.findResponce((String)hash.get("Sentence"))) != null) // ׂ
		{
		    int n_hits = hitResponces.size();
		    int id = (int)((Math.random() * 10000) % n_hits);
		    
		    return generateResponce200(
			    evaluator.evaluateAndUnescape( ((SCKawariResponce)hitResponces.elementAt(id)).script() ),
			    hash);
		}
		else if ((respUnknownEntries = dictionary.findEntry("system.OnResponseUnknown")) != null) // system.OnResponseUnknownGg΂ԂB
		{
		    int n_ue = respUnknownEntries.size();
		    int id = (int)((Math.random() * 10000) % n_ue);
		    
		    return generateResponce200(
			    evaluator.evaluateAndUnescape( (String)respUnknownEntries.elementAt(id) )
			    ,hash);
		}
		else // ꂷ疳204B
		{
		    return "SHIORI/2.0 204 No Content\r\n\r\n";
		}
	    }
	    else // CxgZeX
	    {
		String sentenceToReturn = getaistringrandom();
		return (sentenceToReturn == null ? "SHIORI/2.0 204 No Content\r\n\r\n" : generateResponce200(sentenceToReturn,hash));
	    }
	}
	else if (header.startsWith("GET Word"))
	{
	    String type = (String)hash.get("Type");
	    String sentence = (String)hash.get("Sentence");
	    if (type != null)
	    {
		String target_entry_name = (String)GET_WORD_CONVERT_TABLE.get(type);
		if (target_entry_name == null)
		{
		    return "SHIORI/2.3 400 Bad Request\r\n\r\n";
		}
		
		Vector words = dictionary.findEntry(target_entry_name);
		if (words == null)
		{
		    return "SHIORI/2.0 204 No Content\r\n\r\n";
		}
		
		int n_words = words.size();
		int id = (int)((Math.random() * 10000) % n_words);
		
		return "SHIORI/2.0 200 OK\r\nWord: " + evaluator.evaluateAndUnescape( (String)words.elementAt(id) ) + "\r\n\r\n";
	    }
	    else if (sentence != null)
	    {
		// GET Word with SentencéÂƂ^ʖڂɏC͖B
		return "SHIORI/2.0 204 No Content\r\n\r\n";
	    }
	    else
	    {
		return "SHIORI/2.3 400 Bad Request\r\n\r\n";
	    }
	}
	else if (header.startsWith("GET String"))
	{
	    Vector resources = dictionary.findEntry("resource."+(String)hash.get("ID"));
	    if (resources == null)
	    {
		return "SHIORI/2.0 204 No Content\r\n\r\n";
	    }
	    
	    int n_resources = resources.size();
	    int id = (int)((Math.random() * 10000) % n_resources);
	    
	    return "SHIORI/2.5 200 OK\r\nString: " + evaluator.evaluateAndUnescape( (String)resources.elementAt(id) ) + "\r\n\r\n";
	}
	else if (header.startsWith("NOTIFY OwnerGhostName"))
	{
	    ownerGhostName = (String)hash.get("Ghost");
	    
	    // CxgsB
	    String sysONG = dictionary.findEntryAndGetFirstElement("system.OnNotifyGhost");
	    if (sysONG != null) evaluator.evaluate(sysONG); // ʂ͎̂ĂB
	    
	    return "SHIORI/2.3 204 No Content\r\n\r\n";
	}
	else if (header.startsWith("TEACH"))
	{
	    // ̂ƂؘaTEACHɑ΂200204ԂȂB
	    
	    // Word: -> system.Sentence
	    dictionary.clearEntry("system.Sentence");
	    dictionary.addToken("system.Sentence",(String)hash.get("Word"));
	    
	    Vector sysOnTeach;
	    if ((sysOnTeach = dictionary.findEntry("system.OnTeach")) != null)
	    {
		int n_sor = sysOnTeach.size();
		int id = (int)((Math.random() * 10000) % n_sor);
		
		return generateResponce200(
			    evaluator.evaluateAndUnescape( (String)sysOnTeach.elementAt(id) )
			    ,hash);
	    }
	    else
	    {
		return "SHIORI/2.0 204 No Content\r\n\r\n";
	    }
	}
	else
	{
	    return "SHIORI/2.0 500 Internal Server Error\r\n\r\n";
	}
    }
    
    private String generateResponce200(String sentence,Hashtable requestTable)
    {
	// requestTable̓S[XgԃR~jP[g邽߁B
	
	// sentenceȂ204B
	if (sentence.length() == 0)
	{
	    return "SHIORI/2.0 204 No Content\r\n\r\n";
	}
	
	String sysCom = dictionary.findEntryAndGetFirstElement("system.communicate");
	if (requestTable.get("Age") != null) // Age
	{
	    if (sysCom != null && sysCom.equals("stop"))
	    {
		// system.communicatestopbIB
		return "SHIORI/2.3 200 OK\r\nSentence: "+sentence+"\r\n\r\n";
	    }
	    if (sysCom == null || sysCom.equals( (String)requestTable.get("Sender") ))
	    {
		// system.communicateA̓NGXgSenderƓstopłȂꍇɂ͉B
		try
		{
		    int oldAge = Integer.parseInt( (String)requestTable.get("Age") );
		    
		    StringBuffer resp = new StringBuffer("SHIORI/2.3 200 OK\r\n");
		    resp.append("Sender: "+ownerGhostName+"\r\n");
		    resp.append("To: "+(String)requestTable.get("Sender")+"\r\n");
		    resp.append("Age: "+Integer.toString(oldAge+1)+"\r\n");
		    resp.append("Sentence: "+sentence+"\r\n");
		    final int numOfReferencesToSearch = 8;
		    for (int i = 0;i < numOfReferencesToSearch;i++)
		    {
			String ref = dictionary.findEntryAndGetFirstElement("system.Response"+i);
			if (ref == null) continue;
			
			resp.append("Reference"+i+": "+ref+"\r\n");
		    }
		    resp.append("\r\n"); // X|XB
		    return resp.toString();
		}
		catch (Exception e)
		{
		    return "SHIORI/2.3 400 Bad Request\r\n\r\n";
		}
	    }
	    else
	    {
		// system.communicatestopłłȂANGXgSenderƈقȂꍇ͕ʂ̑ɘbB
		StringBuffer resp = new StringBuffer("SHIORI/2.3 200 OK\r\n");
		resp.append("Sender: "+ownerGhostName+"\r\n");
		resp.append("To: "+sysCom+"\r\n");
		resp.append("Age: 0\r\n");
		resp.append("Sentence: "+sentence+"\r\n");
		final int numOfReferencesToSearch = 8;
		for (int i = 0;i < numOfReferencesToSearch;i++)
		{
		    String ref = dictionary.findEntryAndGetFirstElement("system.Response"+i);
		    if (ref == null) continue;
		    
		    resp.append("Reference"+i+": "+ref+"\r\n");
		}
		resp.append("\r\n"); // X|XB
		return resp.toString();
	    }
	}
	else // Age
	{
	    if (sysCom != null && !sysCom.equals("stop"))
	    {
		// system.communicatełstopłȂVɃS[XgԃR~jP[VJnB
		StringBuffer resp = new StringBuffer("SHIORI/2.3 200 OK\r\n");
		resp.append("Sender: "+ownerGhostName+"\r\n");
		resp.append("To: "+sysCom+"\r\n");
		resp.append("Age: 0\r\n");
		resp.append("Sentence: "+sentence+"\r\n");
		final int numOfReferencesToSearch = 8;
		for (int i = 0;i < numOfReferencesToSearch;i++)
		{
		    String ref = dictionary.findEntryAndGetFirstElement("system.Response"+i);
		    if (ref == null) continue;
		    
		    resp.append("Reference"+i+": "+ref+"\r\n");
		}
		resp.append("\r\n"); // X|XB
		return resp.toString();
	    }
	    else
	    {
		// system.communiate܂stop̂ŒʏSentencê݁B
		return "SHIORI/2.0 200 OK\r\nSentence: "+sentence+"\r\n\r\n";
	    }
	}
    }
}
