/*
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.util.*;

public class SCKawariEvaluator
{
    SCKawari kawari;
    SCKis kis;
    
    public SCKawariEvaluator(SCKawari kawari)
    {
	this.kawari = kawari;
	kis = new SCKis(kawari);
    }
    
    public String evaluate(String str)
    {
	return evaluate(str,false);
    }
    
    public String evaluateAndUnescape(String str)
    {
	return SCKawariDictionary.unescape(evaluate(str,false));
    }
    
    public String unescapeAndEvaluate(String str)
    {
	return evaluate(SCKawariDictionary.unescape(str),false);
    }
    
    private String evaluate(String str,boolean isCommand)
    {
	// $()${}]܂B
	StringBuffer buf = new StringBuffer(str);
	
	if (!isCommand)
	{
	    // R}hlXgĂ\B
	    Vector localdict = new Vector(); // ${[]}p
	    while (true)
	    {
		int comstart = buf.toString().indexOf("$(");
		int refstart = buf.toString().indexOf("${");
		if (comstart == -1 && refstart == -1) break; // ]IB
	    
		if (comstart != -1 && (refstart == -1 || comstart < refstart)) // R}hɌꂽB
		{
		    // lXg̍łOɂR}hoĕ]B
		    int blockstart = comstart;
		    int blockend = buf.toString().indexOf(')',blockstart);
		    
		    int lastFoundNestStart = blockstart;
		    while (true)
		    {
			// u$(vTAblockstart`blockend͈̔͂Ō玟́u)vTB
			int nestStart = buf.toString().indexOf("$(",lastFoundNestStart+1);
			if (nestStart != -1 && nestStart < blockend) // lXgĂ
			{
			    lastFoundNestStart = nestStart;
			    blockend = buf.toString().indexOf(')',blockend+1);
			}
			else
			{
			    break; // blockendm
			}
		    }
		    if (blockend == -1)
		    {
			System.err.println("The Java Clone Of Kawari : Syntax error in script. Missing \")\".");
			return "";
		    }
		    
		    // ŁAblockstart`blockend͈̔͂łÕlXg̃R}hƂȂB
		    buf.replace(blockstart,blockend+1,evaluate(buf.substring(blockstart,blockend+1),true));
		}
		else if (refstart != -1) // t@XɌꂽB
		{
		    int blockstart = refstart;
		    int blockend = buf.toString().indexOf("}",blockstart);
		    
		    int lastFoundNestStart = blockstart;
		    while (true)
		    {
			// u${vTAblockstart`blockend͈̔͂Ō玟́u}vTB
			int nestStart = buf.toString().indexOf("${",lastFoundNestStart+1);
			if (nestStart != -1 && nestStart < blockend) // lXgĂ
			{
			    lastFoundNestStart = nestStart;
			    blockend = buf.toString().indexOf('}',blockend+1);
			}
			else
			{
			    break; // blockendm
			}
		    }
		    if (blockend == -1)
		    {
			System.err.println("The Java Clone Of Kawari : Syntax error in script. Missing \"}\".");
			return "";
		    }
		    // ŁAblockstart`blockend͈̔͂łÕlXg${}ƂȂB
		    String key = evaluate(buf.substring(blockstart+2,blockend),false);
		    
		    try
		    {
			int history = Integer.parseInt(key);
			
			if (history < 0 || history >= localdict.size())
			{
			    // IDs
			    buf.delete(blockstart,blockend+1);
			}
			else
			{
			    buf.replace(blockstart,blockend+1,(String)localdict.elementAt(history));
			}
		    }
		    catch (Exception e) // łȂ
		    {
			Vector items = kawari.dictionary().findEntry(key);
			if (items == null) // Ȃ
			{
			    buf.delete(blockstart,blockend+1);
			    continue;
			}
			
			int id = (int)((Math.random() * 10000) % items.size());
			String subsen = evaluate( (String)items.elementAt(id) , false); // 
			// ĕԂĂKvȕ]͏IĂ͂B
			localdict.add(subsen);
			buf.replace(blockstart,blockend + 1,subsen);
		    }
		}
	    }
	    return buf.toString();
	}
	else
	{
	    // R}h̓lXgĂ\B
	    // u$(vŎn܂u)vŏIĂB
	    String content = buf.substring(2,buf.length()-1);
	    StringBuffer result = new StringBuffer();
	    
	    // ܂̓Xe[ggɕB
	    SCKisTokenizer statements = new SCKisTokenizer(content,";");
	    while (statements.hasMoreTokens())
	    {
		String statement = statements.nextToken().trim(); // Ő󔒂͏B
		//System.out.println("STATEMENT="+statement);
		
		// ɂR}hƈQɕB
		SCKisTokenizer tokens = new SCKisTokenizer(statement," ");
		String command = evaluate(tokens.nextToken(),false);
		Vector params = new Vector();
		if (
		    command.equals("while") || command.equals("until") || 
		    command.equals("if") || command.equals("?"))
		{
		    // Sĕ]ȂB
		    while (tokens.hasMoreTokens())
		    {
			String token = tokens.nextToken();
			params.add(token);
		    }
		}
		else if (command.equals("loop"))
		{
		    // ]ȂB
		    while (tokens.hasMoreTokens())
		    {
			String token = tokens.nextToken();
			if (params.size() == 1)
			{
			    params.add(token);
			}
			else
			{
			    String evaluated = evaluate(token,false);
			    params.add(evaluated);
			}
		    }
		}
		else if (command.equals("foreach"))
		{
		    // O]ȂB
		    while (tokens.hasMoreTokens())
		    {
			String token = tokens.nextToken();
			if (params.size() == 2)
			{
			    params.add(token);
			}
			else
			{
			    String evaluated = evaluate(token,false);
			    params.add(evaluated);
			}
		    }
		}
		else
		{
		    // Sĕ]B
		    while (tokens.hasMoreTokens())
		    {
			String token = tokens.nextToken();
			String evaluated = evaluate(token,false);
			params.add(evaluated);
		    }
		}
		
		// IBs܂B
		if (command.equals("silent"))
		{
		    result.setLength(0); // R}hsilentA܂ł̌ʂ̂ċB
		}
		else
		{
		    //dumpStatement(command,params);
		    String execresult = kis.execute(command,params);
		    if (execresult != null)
		    {
			result.append(execresult);
		    }
		}
	    }
	    
	    return result.toString();
	}
    }
    
    private void dumpStatement(String command,Vector args)
    {
	StringBuffer dump = new StringBuffer("COMMAND="+command);
	final int argc = args.size();
	for (int i = 0;i < argc;i++)
	{
	    dump.append(':'+(String)args.elementAt(i));
	}
	System.out.println(dump.toString());
    }
}
