/*
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.util.Vector;

public class SCKisSyntaxCommands extends SCKisCommands
{
    public SCKisSyntaxCommands(SCKawari kawari)
    {
	super(kawari);
    }
    
    public String syntax_if(Vector params)
    {
	/*
	    @@Fif  1 [2]
	    ߂lF^Ȃ當1AUȂ當2sAUŕ2ȗĂnull
	*/
	String judgement = (String)params.elementAt(0);
	String statement1 = (String)params.elementAt(1);
	String statement2 = (params.size() >= 3 ? (String)params.elementAt(2) : "");
	if (SCKis.evaluateAsBoolean( kawari.evaluator().evaluate(judgement) ))
	{
	    // ^
	    return kawari.evaluator().evaluate(statement1);
	}
	else
	{
	    // U
	    return (params.size() < 3 ? null : kawari.evaluator().evaluate(statement2));
	}
    }
    
    public String syntax_while(Vector params)
    {
	/*
	    @@Fwhile  
	    ߂lF̌ʁ~JԂ
	*/
	StringBuffer result = new StringBuffer();
	String judgement = (String)params.elementAt(0);
	String statement = (String)params.elementAt(1);
	while (SCKis.evaluateAsBoolean( kawari.evaluator().evaluate(judgement) ))
	{
	    result.append( kawari.evaluator().evaluate(statement) );
	}
	return result.toString();
    }
    
    public String until(Vector params)
    {
	/*
	    @@Funtil  
	    ߂lF̌ʁ~JԂ
	*/
	StringBuffer result = new StringBuffer();
	String judgement = (String)params.elementAt(0);
	String statement = (String)params.elementAt(1);
	while (!SCKis.evaluateAsBoolean( kawari.evaluator().evaluate(judgement) ))
	{
	    result.append( kawari.evaluator().evaluate(statement) );
	}
	return result.toString();
    }
    
    public String foreach(Vector params)
    {
	/*
	    @@Fforeach Gg ]Gg 
	    ߂lF̌ʁ~]Gg̑So^Pꐔ
	*/
	StringBuffer result = new StringBuffer();
	String destEntryName = (String)params.elementAt(0);
	Vector pool = kawari.dictionary().findEntry((String)params.elementAt(1));
	String statement = (String)params.elementAt(2);
	
	int n_itemsInPool = pool.size();
	for (int i = 0;i < n_itemsInPool;i++)
	{
	    kawari.dictionary().clearEntry(destEntryName);
	    kawari.dictionary().addToken(destEntryName,(String)pool.elementAt(i));
	    
	    result.append( kawari.evaluator().evaluate(statement) );
	}
	return result.toString();
    }
    
    public String NULL(Vector params)
    {
	/*
	    @@FNULL
	    ߂lF
	*/
	return null;
    }
    
    public String loop(Vector params)
    {
	/*
	    @@Floop  
	    ߂lF̌ʁ~
	*/
	try
	{
	    StringBuffer result = new StringBuffer();
	    int times = Integer.parseInt( (String)params.elementAt(0) );
	    String statement = (String)params.elementAt(1);
	    
	    for (int i = 0;i < times;i++)
	    {
		result.append( kawari.evaluator().evaluate(statement) );
	    }
	    
	    return result.toString();
	}
	catch (Exception e)
	{
	    return null;
	}
    }
    
    public String syntax_randomselect(Vector params)
    {
	/*
	    @@F? 1 2 3 ... n
	    ߂lF1...n܂ł̂ǂꂩ̖߂l
	*/
	int select = (int)((Math.random() * 10000) % params.size());
	return kawari.evaluator().evaluate( (String)params.elementAt(select) );
    }
}
