/*
    $Id: SCMenuAppearanceServer.java,v 1.5 2005/04/28 19:10:28 phonohawk Exp $
    ----------
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2005 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;
import java.util.*;

public class SCMenuAppearanceServer {
    public static final int ALIGN_LEFTTOP = 1;
    public static final int ALIGN_RIGHTTOP = 2;
    public static final int ALIGN_LEFTBOTTOM = 3;
    public static final int ALIGN_RIGHTBOTTOM = 4;
    public static final int ALIGN_TOP = 1000;
    public static final int ALIGN_BOTTOM = 1001;
    
    protected static final Hashtable ALIGN_TABLE;
    static {
	ALIGN_TABLE = new Hashtable();
	ALIGN_TABLE.put("lefttop",new Integer(ALIGN_LEFTTOP));
	ALIGN_TABLE.put("righttop",new Integer(ALIGN_RIGHTTOP));
	ALIGN_TABLE.put("leftbottom",new Integer(ALIGN_LEFTBOTTOM));
	ALIGN_TABLE.put("rightbottom",new Integer(ALIGN_RIGHTBOTTOM));
	ALIGN_TABLE.put("top",new Integer(ALIGN_TOP));
	ALIGN_TABLE.put("bottom",new Integer(ALIGN_BOTTOM));
    }
    
    SCSession session;
    
    boolean hasImage = false;
    NSImage bgImage;
    NSImage fgImage;
    NSImage sakuraSideImage;
    NSImage keroSideImage;
    NSColor bgFontColor;
    NSColor fgFontColor;
    NSColor disabledColor;
    NSColor separatorColor;
    int align_bg;
    int align_fg;
    int align_side;
    NSColor topLeftColorOfSakuraSideImage;
    NSColor topLeftColorOfKeroSideImage;
    NSColor bottomRightColorOfSakuraSideImage;
    NSColor bottomRightColorOfKeroSideImage;
    NSColor topLeftColorOfBgImage;
    NSColor topLeftColorOfFgImage;
    NSColor bottomRightColorOfBgImage;
    NSColor bottomRightColorOfFgImage;
    
    public SCMenuAppearanceServer(SCSession session) {
	this.session = session;
	
	SCDescription desc;
	File dir;
	
	SCDescription shell_desc = session.getCurrentShell().getDescManager();
	if (shell_desc.exists("menu.foreground.bitmap.filename") ||
	    new File(session.getCurrentShell().getRootDir(),"menu_background.png").exists()) { // VFɂƌ􂷁B
	    desc = shell_desc;
	    dir = session.getCurrentShell().getRootDir();
	}
	else { // S[Xgɂƌ􂷁B
	    desc = session.getMasterSpirit().getDescManager();
	    dir = session.getMasterSpirit().getRootDir();
	}
		
	String bg_image_filename;
	String fg_image_filename;
	String side_image_filename;
	
	if (desc.exists("menu.background.bitmap.filename")) {
	    bg_image_filename = backslash_to_slash(
		desc.getStrValue("menu.background.bitmap.filename"));
	}
	else {
	    bg_image_filename = "menu_background.png";
	}
	
	if (desc.exists("menu.foreground.bitmap.filename")) {
	    fg_image_filename = backslash_to_slash(
		desc.getStrValue("menu.foreground.bitmap.filename"));
	}
	else {
	    fg_image_filename = "menu_foreground.png";
	}
	
	if (desc.exists("menu.sidebar.bitmap.filename")) {
	    side_image_filename = backslash_to_slash(
		desc.getStrValue("menu.sidebar.bitmap.filename"));
	}
	else {
	    side_image_filename = "menu_sidebar.png";
	}
	
	File bgImgFile = new File(dir,bg_image_filename);
	File fgImgFile = new File(dir,fg_image_filename);
	
	File sideImgFile = new File(dir,side_image_filename);
	File sakuraSideImgFile = new File(dir,side_image_filename.substring(0,side_image_filename.length()-4)+"1.png");
	File keroSideImgFile = new File(dir,side_image_filename.substring(0,side_image_filename.length()-4)+"0.png");
	
	if (bgImgFile.exists() && fgImgFile.exists() &&
	    (sideImgFile.exists() || (sakuraSideImgFile.exists() && keroSideImgFile.exists()))) {
	    hasImage = true;
	    
	    bgImage = new NSImage(bgImgFile.getPath(),true);
	    fgImage = new NSImage(fgImgFile.getPath(),true);
	    if (sideImgFile.exists()) {
		sakuraSideImage = keroSideImage = new NSImage(sideImgFile.getPath(),true);
		topLeftColorOfSakuraSideImage = topLeftColorOfKeroSideImage = SCImageUtils.getTopLeftColor(sakuraSideImage);
	    }
	    else {
		sakuraSideImage = new NSImage(sakuraSideImgFile.getPath(),true);
		keroSideImage = new NSImage(keroSideImgFile.getPath(),true);
		
		topLeftColorOfSakuraSideImage = SCImageUtils.getTopLeftColor(sakuraSideImage);
		topLeftColorOfKeroSideImage = SCImageUtils.getTopLeftColor(keroSideImage);
	    }
	    
	    topLeftColorOfBgImage = SCImageUtils.getTopLeftColor(bgImage);
	    topLeftColorOfFgImage = SCImageUtils.getTopLeftColor(fgImage);
	    bottomRightColorOfBgImage = SCImageUtils.getBottomRightColor(bgImage);
	    bottomRightColorOfFgImage = SCImageUtils.getBottomRightColor(fgImage);
	}
	
	if (desc.exists("menu.background.font.color.r")) {
	    float r,g,b;
	    r = desc.getIntValue("menu.background.font.color.r") / 255.0f;
	    g = desc.getIntValue("menu.background.font.color.g") / 255.0f;
	    b = desc.getIntValue("menu.background.font.color.b") / 255.0f;
	    bgFontColor = NSColor.colorWithCalibratedRGB(r,g,b,1.0f);
	    disabledColor = NSColor.colorWithCalibratedRGB(r,g,b,0.5f);
	}
	else {
	    bgFontColor = NSColor.blackColor();
	    disabledColor = NSColor.colorWithCalibratedRGB(0.0f,0.0f,0.0f,0.5f);
	}
	
	if (desc.exists("menu.foreground.font.color.r")) {
	    float r,g,b;
	    r = desc.getIntValue("menu.foreground.font.color.r") / 255.0f;
	    g = desc.getIntValue("menu.foreground.font.color.g") / 255.0f;
	    b = desc.getIntValue("menu.foreground.font.color.b") / 255.0f;
	    fgFontColor = NSColor.colorWithCalibratedRGB(r,g,b,1.0f);
	}
	else {
	    fgFontColor = NSColor.whiteColor();
	}
	
	if (desc.exists("menu.separator.color.r")) {
	    float r,g,b;
	    r = desc.getIntValue("menu.separator.color.r") / 255.0f;
	    g = desc.getIntValue("menu.separator.color.g") / 255.0f;
	    b = desc.getIntValue("menu.separator.color.b") / 255.0f;
	    separatorColor = NSColor.colorWithCalibratedRGB(r,g,b,1.0f);
	}
	else {
	    separatorColor = NSColor.whiteColor();
	}
	
	if (desc.exists("menu.background.alignment")) {
	    Integer i = (Integer)ALIGN_TABLE.get(desc.getStrValue("menu.background.alignment"));
	    align_bg = (i == null ? ALIGN_RIGHTTOP : i.intValue());
	}
	else {
	    align_bg = ALIGN_RIGHTTOP;
	}
	
	if (desc.exists("menu.foreground.alignment")) {
	    Integer i = (Integer)ALIGN_TABLE.get(desc.getStrValue("menu.foreground.alignment"));
	    align_fg = (i == null ? ALIGN_RIGHTTOP : i.intValue());
	}
	else {
	    align_fg = ALIGN_RIGHTTOP;
	}
	
	if (desc.exists("menu.sidebar.alignment")) {
	    Integer i = (Integer)ALIGN_TABLE.get(desc.getStrValue("menu.sidebar.alignment"));
	    align_side = (i == null ? ALIGN_BOTTOM : i.intValue());
	}
	else {
	    align_side = ALIGN_BOTTOM;
	}
    }

    public static String backslash_to_slash(String src) {
	if (src.indexOf('\\') != -1) {
	    StringBuffer buf = new StringBuffer(src);
	    int pos;
	    while ((pos = buf.toString().indexOf('\\')) != -1) {
		buf.setCharAt(pos,'/');
	    }
	    return buf.toString();
	}
	else {
	    return src;
	}
    }
    
    public boolean hasImage() {return hasImage;}
    public NSImage getBgImage() {return bgImage;}
    public NSImage getFgImage() {return fgImage;}
    public NSImage getSideImage(int scope) {return scope == 0 ? sakuraSideImage : keroSideImage;}
    public NSColor getBgFontColor() {return bgFontColor;}
    public NSColor getFgFontColor() {return fgFontColor;}
    public NSColor getDisabledColor() {return disabledColor;}
    public NSColor getSeparatorColor() {return separatorColor;}
    public int getAlignmentOfBg() {return align_bg;}
    public int getAlignmentOfFg() {return align_fg;}
    public int getAlignmentOfSide() {return align_side;}
    public NSColor getTopLeftColorOfSideImage(int scope) {
	return scope == 0 ? topLeftColorOfSakuraSideImage : topLeftColorOfKeroSideImage;}
    public NSColor getBottomRightColorOfSideImage(int scope) {
	return scope == 0 ? bottomRightColorOfSakuraSideImage : bottomRightColorOfKeroSideImage;}
    public NSColor getTopLeftColorOfBgImage() {return topLeftColorOfBgImage;}
    public NSColor getTopLeftColorOfFgImage() {return topLeftColorOfFgImage;}
    public NSColor getBottomRightColorOfBgImage() {return bottomRightColorOfBgImage;}
    public NSColor getBottomRightColorOfFgImage() {return bottomRightColorOfFgImage;}
}
