/*  $Id: SCShellContextMenuMaker.java,v 1.5 2005/04/28 19:10:28 phonohawk Exp $
	
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001-2005 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.util.*;
import java.net.*;

public class SCShellContextMenuMaker {
    SCSession session;
    
    NSMenu hontaiMenu;
    NSMenu unyuuMenu;
    SCContextMenuView hontaiMenuView;
    SCContextMenuView unyuuMenuView;
    
    NSMenuItem sakura_recommended;
    NSMenuItem portal;
    NSMenuItem dev_interfaces;
    NSMenuItem change_ghost;
    NSMenuItem change_shell;
    NSMenuItem change_balloon;
    NSMenuItem network_update;
    NSMenuItem vanish;
    NSMenuItem headline_sensor;
    NSMenuItem about_ghost;
    NSMenuItem about_shell;
    NSMenuItem disappear;
    
    NSMenuItem unyuu_recommended;
    
    private final NSSelector selector_urlMenuItemSelected;
    private final NSSelector selector_hlsensorMenuItemSelected;
    
    public SCShellContextMenuMaker(SCSession s) {
	session = s;
		
	selector_urlMenuItemSelected = new NSSelector("urlMenuItemSelected",new Class[] {Object.class});
	selector_hlsensorMenuItemSelected = new NSSelector("hlsensorMenuItemSelected",new Class[] {Object.class});
		
	hontaiMenu = new NSMenu();
	hontaiMenu.setAutoenablesItems(false);
	hontaiMenuView = new SCContextMenuView(session, 0, true);
	hontaiMenu.setMenuRepresentation(hontaiMenuView);
	
	// ̂
	sakura_recommended = new NSMenuItem("",null,"");
	hontaiMenu.addItem(sakura_recommended);
	
	// |[^
	portal = new NSMenuItem("",null,"");
	hontaiMenu.addItem(portal);
	
	// ؂
	addSeparator(hontaiMenu);
	
	// S[XgύX
	change_ghost = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.changeghost"),null,"");
	hontaiMenu.addItem(change_ghost);
	
	// VF
	change_shell = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.shell"),null,"");
	hontaiMenu.addItem(change_shell);
	
	// o[
	change_balloon = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.balloon"),null,"");
	hontaiMenu.addItem(change_balloon);
	
	// ؂
	addSeparator(hontaiMenu);
	
	// lbg[NXV
	network_update = new NSMenuItem("",null,"");
	network_update.setAction(new NSSelector("networkUpdateSelected",new Class[] {Object.class}));
	network_update.setTarget(this);
	hontaiMenu.addItem(network_update);
	
	// Œʍ
	NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
	if (defaults.integerForKey("misc.always_show_vanish") == 1 ||
	    session.getStringFromShiori("vanishbuttonvisible") == null ||
	    !session.getStringFromShiori("vanishbuttonvisible").equals("0")) {
	    vanish = new NSMenuItem("",null,"");
	    vanish.setAction(new NSSelector("vanishSelected",new Class[] {Object.class}));
	    vanish.setTarget(this);
	    hontaiMenu.addItem(vanish);
	}
	else {
	    vanish = null;
	}
		
	// ؂
	addSeparator(hontaiMenu);
	
	// JpC^[tFCX
	if (defaults.integerForKey("misc.show_dev_interfaces") == 1) {
	    dev_interfaces = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.development"),null,"");
	    hontaiMenu.addItem(dev_interfaces);
	    
	    NSMenu submenu = makeDevInterfacesSubMenu();
	    SCContextMenuView submenuView = new SCContextMenuView(session, 0);
	    submenu.setMenuRepresentation(submenuView);
	    dev_interfaces.setSubmenu(submenu);
	    setCustumCells(submenuView, 0);
	    
	    // ؂
	    addSeparator(hontaiMenu);
	}
	
	// wbhCZT
	headline_sensor = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.hlsensor"),null,"");
	hontaiMenu.addItem(headline_sensor);
	
	// ؂
	addSeparator(hontaiMenu);
	
	// ̃S[Xgɂ...
	about_ghost = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.aboutghost"),null,"");
	about_ghost.setEnabled(session.getReadmeFile().exists());
	about_ghost.setAction(new NSSelector("aboutThisGhost",new Class[] {Object.class}));
	about_ghost.setTarget(this);
	hontaiMenu.addItem(about_ghost);
	
	// ̃VFɂ...
	about_shell = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.aboutshell"),null,"");
	about_shell.setAction(new NSSelector("aboutThisShell",new Class[] {Object.class}));
	about_shell.setTarget(this);
	hontaiMenu.addItem(about_shell);
	
	// ؂
	addSeparator(hontaiMenu);
		
	// 
	disappear = new NSMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.close"),null,"");
	disappear.setAction(new NSSelector("closeSelected",new Class[] {Object.class}));
	disappear.setTarget(this);
	hontaiMenu.addItem(disappear);
	
	// ȏSĂ̍ڂɃJX^ZݒB
	setCustumCells(hontaiMenuView, 0);
	
	
	unyuuMenu = new NSMenu();
	unyuuMenu.setAutoenablesItems(false);
	unyuuMenuView = new SCContextMenuView(session,1,true);
	unyuuMenu.setMenuRepresentation(unyuuMenuView);
	
	// ɂイ̂
	unyuu_recommended = new NSMenuItem("",null,"");
	unyuuMenu.addItem(unyuu_recommended);
	
	// ȏSĂ̍ڂɃJX^ZݒB
	setCustumCells(unyuuMenuView,1);
    }
    
    public void reconstructMenu(int scope) {
	// j[\ĂԂɍč\ȂƁB듮삵܂B
	if (scope == 0) {
	    // 
	    {
		String title = session.getStringFromShiori("sakura.recommendbuttoncaption");
		// 擾oȂ΁u%selfnamê߁vƂB
		if (title == null || title.length() == 0) {
		    title = SCStringsServer.getStrFromMainDic("contextmenu.close",new String[] {session.getSelfName()});
		}
		sakura_recommended.setTitle(title);
		setTable(sakura_recommended,"sakura.recommendsites",scope);
	    }
	    
	    // |[^
	    {
		String title = session.getStringFromShiori("sakura.portalbuttoncaption");
		// 擾oȂ΁u|[^vƂB
		if (title == null || title.length() == 0) {
		    title = SCStringsServer.getStrFromMainDic("contextmenu.portal");
		}
		portal.setTitle(title);
		setTable(portal,"sakura.portalsites",scope);
	    }
	    
	    // S[XgύX
	    {
		if (session.isStatusClosing()) {
		    change_ghost.setEnabled(false);
		}
		else {
		    NSMenu submenu = SCGhostManager.sharedGhostManager().makeGhostMenu(session);
		    SCContextMenuView submenuView = new SCContextMenuView(session, scope);
		    submenu.setMenuRepresentation(submenuView);
		    change_ghost.setSubmenu(submenu);
		    
		    setCustumCells(submenuView, scope);
		}
	    }
	    
	    // VF
	    {
		if (session.isShellChangingSessionRunningNow() ||
		    session.isStatusClosing() ||
		    session.isNetworkUpdaterRunningNow()) {
		    change_shell.setEnabled(false);
		}
		else {
		    NSMenu submenu = SCGhostManager.sharedGhostManager().makeShellMenu(session);
		    SCContextMenuView submenuView = new SCContextMenuView(session, scope);
		    submenu.setMenuRepresentation(submenuView);
		    change_shell.setSubmenu(submenu);
		    
		    setCustumCells(submenuView, scope);
		    
		    change_shell.setEnabled(true);
		}
	    }
	    
	    // o[
	    {
		NSMenu submenu = SCGhostManager.sharedGhostManager().makeBalloonMenu(session);
		SCContextMenuView submenuView = new SCContextMenuView(session, scope);
		submenu.setMenuRepresentation(submenuView);
		change_balloon.setSubmenu(submenu);
		
		setCustumCells(submenuView, scope);
	    }
	    
	    // ւ
	    /*NSMenuItem item_kisekae_submenu = new NSMenuItem("\u7740\u305b\u66ff\u3048",null,"");
	      NSMenu kisekae_submenu = session.getCurrentShell().getSeriko().getBindCenter().getBindMenu();
	      if (kisekae_submenu == null)
	      {
	      item_kisekae_submenu.setEnabled(false);
	      }
	      else
	      {
	      item_kisekae_submenu.setSubmenu(kisekae_submenu);
	      }
	      hontaiMenu.addItem(item_kisekae_submenu);*/
	    
	    // lbg[NXV
	    {
		String title = session.getStringFromShiori("updatebuttoncaption");
		if (title == null || title.length() == 0) {
		    title = SCStringsServer.getStrFromMainDic("contextmenu.networkupdate");
		}
		network_update.setTitle(title);
		
		if (session.getStringFromShiori("homeurl") != null &&
		    !session.isNetworkUpdaterRunningNow() &&
		    !session.isStatusClosing()) {
		    network_update.setEnabled(true);
		}
		else {
		    network_update.setEnabled(false);
		}
	    }
	    
	    // Œʍ
	    if (vanish != null) {
		String title = session.getStringFromShiori("vanishbuttoncaption");
		if (title == null || title.length() == 0) {
		    title = SCStringsServer.getStrFromMainDic("contextmenu.vanish");
		}
		vanish.setTitle(title);
		
		if (!session.isNetworkUpdaterRunningNow() &&
		    !session.isStatusClosing()) {
		    vanish.setEnabled(true);
		}
		else {
		    vanish.setEnabled(false);
		}
	    }
	    
	    // wbhCZT
	    {
		setHeadlineSensorTable(headline_sensor,scope);
	    }
	    
	    // ̃VFɂ...
	    {
		about_shell.setEnabled(session.getCurrentShell().getReadmeFile().exists());
	    }
	    
	    // 
	    {
		if (!session.isStatusClosing() &&
		    !session.isNetworkUpdaterRunningNow()) {
		    disappear.setEnabled(true);
		}
		else {
		    disappear.setEnabled(false);
		}
	    }
	}
	else if (scope == 1) {
	    // ɂイ̂
	    {
		String title = session.getStringFromShiori("kero.recommendbuttoncaption");
		// 擾oȂ΁u%keronamê߁vƂB
		if (title == null) {
		    title = SCStringsServer.getStrFromMainDic("contextmenu.close",new String[] {session.getKeroName()});
		}
		unyuu_recommended.setTitle(title);
		setTable(unyuu_recommended,"kero.recommendsites",scope);
	    }
	}
    }
    
    protected void setCustumCells(NSMenuView menuView, int scope) {
	NSMenu menu = menuView.menu();
	final int n_items = menu.numberOfItems();
	for (int i = 0;i < n_items;i++) {
	    SCContextMenuItemCell mic = new SCContextMenuItemCell(session, scope);
	    mic.setMenuItem(menu.itemAtIndex(i));
	    menuView.setMenuItemCellForItemAtIndex(mic,i);
	}
    }
    
    protected void addSeparator(NSMenu menu) {
	NSMenuItem item = (NSMenuItem)NSMenuItem.protocolSeparatorItem();
	hontaiMenu.addItem(item);
    }
    
    private void setTable(NSMenuItem target, String resourceType, int scope) {
	String rcmTable = session.getStringFromShiori(resourceType);
	// 擾oȂdisableB
	if (rcmTable == null || rcmTable.length() == 0) {
	    target.setEnabled(false);
	}
	else {
	    try {
		NSMenu submenu = new NSMenu();
		submenu.setAutoenablesItems(false);
		SCContextMenuView submenuView = new SCContextMenuView(session,scope);
		submenu.setMenuRepresentation(submenuView);
		
		// ܂0x02ŕ
		StringTokenizer rows = new StringTokenizer(rcmTable,"\u0002");
		while (rows.hasMoreTokens()) {
		    // 0x01ŕ
		    StringTokenizer columns = new StringTokenizer(rows.nextToken(),"\u0001");
		    
		    String name = columns.nextToken();
		    if (name.equals("-")) {
			submenu.addItem((NSMenuItem)NSMenuItem.protocolSeparatorItem());
		    }
		    else {
			String url = columns.nextToken();
			String banner = (columns.hasMoreTokens() ? columns.nextToken() : null);
			URLMenuItem item = new URLMenuItem(name,url,banner);
			item.setAction(selector_urlMenuItemSelected);
			item.setTarget(this);
			item.setImage(SCFoundation.sharedFoundation().getBannerServer().getBanner(banner));
			submenu.addItem(item);
			SCContextMenuItemCell mic = new SCContextMenuItemCell(session,scope,SCContextMenuItemCell.ALIGN_RIGHT);
			mic.setMenuItem(item);
			submenuView.setMenuItemCellForItemAtIndex(mic,submenu.numberOfItems()-1);
		    }
		}
		
		target.setSubmenu(submenu);
	    }
	    catch (Exception e) {
		target.setEnabled(false);
	    }
	}
    }
    
    private void setHeadlineSensorTable(NSMenuItem target,int scope) {
	NSMenu submenu = new NSMenu();
	submenu.setAutoenablesItems(false);
	SCContextMenuView submenuView = new SCContextMenuView(session,scope);
	submenu.setMenuRepresentation(submenuView);
	
	SCPlugin[] sensors = SCFoundation.sharedFoundation().getPluginManager().getPlugins("hlsensor");
	if (sensors.length == 0) {
	    target.setEnabled(false);
	}
	else {
	    target.setEnabled(true);
	    
	    for (int i = 0;i < sensors.length;i++) {
		SCHLSensorPlugin sensor = (SCHLSensorPlugin)sensors[i];
		
		SCHLSensorResponse resp = sensor.request("GET Description HEADLINE/1.0\r\nSender: "+SCFoundation.STRING_FOR_SENDER+"\r\n\r\n");
		String title = resp.getHeader("Name");
		
		HLSensorMenuItem item = new HLSensorMenuItem(title,sensor);
		item.setAction(selector_hlsensorMenuItemSelected);
		item.setTarget(this);
		submenu.addItem(item);
		SCContextMenuItemCell mic = new SCContextMenuItemCell(session,scope,SCContextMenuItemCell.ALIGN_RIGHT);
		mic.setMenuItem(item);
		submenuView.setMenuItemCellForItemAtIndex(mic,submenu.numberOfItems()-1);
	    }
	    
	    target.setSubmenu(submenu);
	}
    }
    
    protected NSMenu makeDevInterfacesSubMenu() {
	NSMenu submenu = new NSMenu();
	submenu.setAutoenablesItems(false);
	
	URLMenuItem item;
	
	// SHIORI/MAKOTO[h
	item = new URLMenuItem(SCStringsServer.getStrFromMainDic("contextmenu.development.reload"),null,"");
	item.setAction(new NSSelector("reloadShioriAndMakoto",new Class[] {Object.class}));
	item.setTarget(this);
	submenu.addItem(item);
	
	return submenu;
    }
    public void reloadShioriAndMakoto(Object sender) {
	session.getMasterSpirit().reloadAll();
    }
    
    public NSMenu getMenu(int scope) {
	// xč\ɌĂяonullԂ܂B
	if (scope == 0) {
	    return hontaiMenu;
	}
	else if (scope == 1) {
	    return unyuuMenu;
	}
	else {
	    return null;
	}
    }
    
    public void networkUpdateSelected(Object sender) {
	session.doNetworkUpdate();
    }
    
    public void closeSelected(Object sender) {
	SCGhostManager.sharedGhostManager().closeGhost(session);
    }
    
    public void vanishSelected(Object sender) {
	session.openVanishDialog();
    }
    
    public void urlMenuItemSelected(Object sender) {
	if (!(sender instanceof URLMenuItem)) return;
	
	URLMenuItem mItem = (URLMenuItem)sender;
	try{ NSWorkspace.sharedWorkspace().openURL(new URL(mItem.getSiteUrl())); }catch(Exception e){}
    }
    
    public void hlsensorMenuItemSelected(Object sender) {
	if (!(sender instanceof HLSensorMenuItem)) return;
	
	session.doHLSensing(((HLSensorMenuItem)sender).getPlugin());
    }
    
    public void aboutThisGhost(Object sender) {
	NSWorkspace.sharedWorkspace().openFile(session.getReadmeFile().getPath(),null,true);
	//new SCDocumentWindowController(session.getReadmeFile(),SCStringsServer.getStrFromMainDic("contextmenu.aboutghost")).showWindow(null);
    }
    
    public void aboutThisShell(Object sender) {
	NSWorkspace.sharedWorkspace().openFile(session.getCurrentShell().getReadmeFile().getPath(),null,true);
	//new SCDocumentWindowController(session.getCurrentShell().getReadmeFile(),SCStringsServer.getStrFromMainDic("contextmenu.aboutshell")).showWindow(null);
    }
    
    private class URLMenuItem extends NSMenuItem {
	String site_name;
	String site_url;
	String banner_url;
	
	public URLMenuItem(String site_name,String site_url,String banner_url) {
	    super(site_name,null,"");
	    
	    this.site_name = site_name;
	    this.site_url = site_url;
	    this.banner_url = banner_url;
	}
	
	public final String getSiteName() {return site_name;}
	public final String getSiteUrl() {return site_url;}
	public final String getBannerUrl() {return banner_url;}
    }
    
    private class HLSensorMenuItem extends NSMenuItem {
	String site_name;
	SCHLSensorPlugin plugin;
	
	public HLSensorMenuItem(String site_name,SCHLSensorPlugin plugin) {
	    super(site_name,null,"");
	    
	    this.site_name = site_name;
	    this.plugin = plugin;
	}
	
	public final String getSiteName() {return site_name;}
	public final SCHLSensorPlugin getPlugin() {return plugin;}
    }
}
