/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001 phonohawkput

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya;
import aya.*;
import aya.node.*;
import aya.eval.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;

public class Aya
{
	public static final String CHARSET = "SJIS";
	
	File ghost_dir;
	Dictionary dic;
	GlobalNamespace global_namespace;
	SystemCall syscall;
	
	String lastreq_method;
	String lastreq_protocol;
	Vector lastreq_headers_v; // String[]; [0]:key [1]:value
	Hashtable lastreq_headers_h; // {key => value}
	
	public Aya(File ghost_dir,File shiori_dll)
	{
		this.ghost_dir = ghost_dir;
		this.dic = new Dictionary(this);
		this.global_namespace = new GlobalNamespace();
		this.syscall = new SystemCall(this);
		
		this.lastreq_method = null;
		this.lastreq_protocol = null;
		this.lastreq_headers_v = new Vector();
		this.lastreq_headers_h = new Hashtable();
		
		// .ayajava-savedata.txtがあれば、aya.txtより先に読んで @@@SAVEDATA@@@ を評価する。
		File savedata = new File(ghost_dir,".ayajava-savedata.txt");
		if (savedata.exists()) {
			try {
				dic.loadFile(savedata);
				Function f_savedata = dic.getFunction("@@@SAVEDATA@@@");
				if (f_savedata != null) {
					f_savedata.eval(null);
					
					// 評価したら、この関数はもう要らないので消す。
					dic.undefFunction("@@@SAVEDATA@@@");
				}
			}
			catch (Exception e) {
				System.err.println("Couldn't load savedata.\n");
				e.printStackTrace();
			}
		}
		
		// aya.dllならaya.txtを、shamrock.dllならshamrock.txtを読む。
		Matcher m = Pattern.compile("^(.+?)\\.[^\\.]+$").matcher(shiori_dll.getName());
		String bootstrapfile;
		if (m.find()) {
			bootstrapfile = m.group(1)+".txt";
		}
		else {
			bootstrapfile = "aya.txt";
		}
		
		// aya.txtがあれば読む。
		File aya_txt = new File(ghost_dir,bootstrapfile);
		if (aya_txt.exists())
		{
			loadAyaConfigFile(aya_txt);
		}
		
		// OnLoadを呼ぶ。
		// 唯一のリクエストヘッダである Path ヘッダからは、aya.dll のフルパスを得ることができる。
		try {
			Function onload = dic.getFunction("OnLoad");
			if (onload != null) {
				lastreq_method = "";
				lastreq_protocol = "";
				
				File fullpath = new File(ghost_dir,"aya.dll");
				lastreq_headers_v.add(new String[] { "Path" , fullpath.getAbsolutePath() });
				lastreq_headers_h.put("Path",fullpath.getAbsolutePath());
			
				onload.eval(null);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public GlobalNamespace getGlobalNamespace() {
		return global_namespace;
	}
	
	public Dictionary getDictionary() {
		return dic;
	}
	
	public SystemCall getSystemCall() {
		return syscall;
	}
	
	public String getMethodOfLastRequest() {
		return lastreq_method;
	}
	
	public String getProtocolOfLastRequest() {
		return lastreq_protocol;
	}
	
	public String getKeyOfLastRequest(int index) {
		if (index >= lastreq_headers_v.size()) {
			return null;
		}
		else {
			String[] entry = (String[])lastreq_headers_v.get(index);
			return entry[0];
		}
	}
	
	public String getValueOfLastRequest(int index) {
		if (index >= lastreq_headers_v.size()) {
			return null;
		}
		else {
			String[] entry = (String[])lastreq_headers_v.get(index);
			return entry[1];
		}
	}
	
	public String getValueOfLastRequest(String key) {
		return (String)lastreq_headers_h.get(key);
	}
	
	protected void loadAyaConfigFile(File confFile)
    {
		try{
			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(confFile),CHARSET));
			while (true)
			{
				String line = br.readLine();
				if (line == null) break; // EOF

				int doubleSlash_pos = line.indexOf("//");
				if (doubleSlash_pos != -1)
				{
					line = line.substring(0,doubleSlash_pos);
				}

				if (line.length() == 0) continue;

				int comma_pos = line.indexOf(',');
				if (comma_pos == -1) continue;
				String key = line.substring(0,comma_pos).trim();
				String value = line.substring(comma_pos+1).trim();

				evaluateAyaConfDefinition(key,value);
			}

			br.close();
		}catch (Exception e){
			System.err.println("SCAya: exception occurerd in loading Aya Config File (aya.txt).");
			e.printStackTrace();
		}
    }
    protected void evaluateAyaConfDefinition(String key,String value)
    {
		if (key.equals("dic"))
		{
			// 辞書を読み込む
			File dicfile = new File(ghost_dir,convertWinPathToUnixOne(value));
			dic.loadFile(dicfile);
		}
    }
    public static String convertWinPathToUnixOne(String path)
    {
		// Windowsのパス区切り￥を、Unixのパス区切り/に置き換えます。
		StringBuffer buf = new StringBuffer(path);
		while (true)
		{
			int en_pos = buf.toString().indexOf('\\');
			if (en_pos == -1) en_pos = buf.toString().indexOf('\u00a5');
			if (en_pos == -1) break; // 終了

			buf.setCharAt(en_pos,'/');
		}
		return buf.toString();
    }
	
	public static boolean checkIfUsable(File ghost_dir,File shiori_dll)
    {
		// 手抜き
		File aya_txt = new File(ghost_dir,"aya.txt");
		if (aya_txt.exists()) return true;
		File aya_dll = new File(ghost_dir,"aya.dll");
		if (aya_dll.exists()) return true;
		
		// shiori.dllにaya.dllの文字列が含まれていたらtrue。
		return (findData(shiori_dll,"aya.dll".getBytes()) != -1);
    }
	
	public static int findData(File f,byte[] data)
    {
		// ファイルからバイト列を検索し、見つかればそれが始まるオフセットを、見つからなければ-1を返します。
		// 現在の実装ではファイルを丸ごとメモリに読み込んでしまうので、あまりに巨大なファイルに対しては利用不可能です。
		if (!f.exists()) {
			return -1;
		}
		
		try
		{
			FileInputStream fis = new FileInputStream(f);
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buf = new byte[512];
			while (true)
			{
				int count = fis.read(buf);
				if (count == -1) break; // eof
				
				baos.write(buf,0,count);
			}
			byte[] world = baos.toByteArray();
			
			return findData(world,data);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return -1;
		}
    }
	public static int findData(byte[] world,byte[] data) {
		// world内からdataを検索し、見つかればそのオフセットを、見つからなければ-1を返します。
		// Boyer Moore法を用いる。
		int data_len = data.length;
		int[] skip = new int[256];
		for (int i = 0; i < 256; i++) {
			skip[i] = data_len;
		}
		for (int i = 0; i < data_len-1; i++) {
			// byte -> -128 から +127 まで。
			// -1は255に、-2は254に、…、-128は128に対応する。
			// つまり、負の範囲ではそれぞれ(256 + n)に対応する。
			skip[(data[i] < 0 ? 256 + data[i] : data[i])] = data_len - i - 1;
		}
		int limit = world.length - data.length;
		for (int i = 0; i <= limit; i += skip[(world[i+data_len-1] < 0 ? 256 + world[i+data_len-1] : world[i+data_len-1])]) {
			if (world[i+data_len-1] != data[data_len-1]) {
				continue;
			}
			
			boolean matched = true;
			for (int j = 0; j < data_len; j++) {
				if (world[i+j] != data[j]) {
					matched = false;
					break;
				}
			}
			if (matched) {
				return i;
			}
		}
		return -1;
	}
	
	public void terminate()
    {
		// OnUnloadを読んでから、
		try {
			Function onunload = dic.getFunction("OnUnload");
			if (onunload != null) {
				onunload.eval(null);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		// 大域的名前空間をファイル .ayajava-savedata.txt に保存する。
		global_namespace.saveToFile(new File(ghost_dir,".ayajava-savedata.txt"));
    }
	
	public static String getModuleName()
    {
		return "Aya";
    }
	
	static final Pattern pat_req_command = Pattern.compile("^(.+?)\\s+(.+)$");
	static final Pattern pat_req_header = Pattern.compile("^(.+?)\\s*:\\s*(.+)$");
	public String request(String req)
    {
		BufferedReader br = new BufferedReader(new StringReader(req));

		// まずはヘッダを読む
		String header = null;
		try{ header = br.readLine(); }catch (Exception e){}
		
		// メソッドとプロトコルに分解
		Matcher mat;
		if ((mat = pat_req_command.matcher(header)).find()) {
			lastreq_method = mat.group(1);
			lastreq_protocol = mat.group(2);
		}
		else {
			return "SHIORI/3.0 400 Bad Request\r\n\r\n";
		}

		// リクエスト全体を読む
		lastreq_headers_v.clear();
		lastreq_headers_h.clear();
		String line = null;
		while (true)
		{
			try{ line = br.readLine(); }catch (Exception e){}
			if (line == null) break;
			
			if ((mat = pat_req_header.matcher(line)).find()) {
				String[] entry = new String[] { mat.group(1) , mat.group(2) };
				
				lastreq_headers_v.add(entry);
				lastreq_headers_h.put(entry[0],entry[1]);
			}
		}
		
		//String id = (String)lastreq_headers_h.get("ID");
		//System.out.println(id);

		try {
			Function onrequest = dic.getFunction("OnRequest");
			if (onrequest != null) {
				return onrequest.eval(null).getString();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return "SHIORI/3.0 500 Internal Server Error\r\n\r\n";
	}
}
