/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya;
import java.io.*;

public class DataDecodingInputStream extends FilterInputStream
{
	public DataDecodingInputStream(InputStream is)
	{
		super(is);
	}
	
	public int read() throws IOException
	{
		int int_data = in.read();
		if (int_data == -1) return -1;
		
		byte byte_data = (byte)((int_data & 0xff) ^ (byte)0xd2);
		return rollByte(byte_data,3);
	}
	
	public int read(byte[] b,int off,int len) throws IOException
	{
		final int n_read = in.read(b,off,len);
		if (n_read == -1) return -1;
		
		for (int i = off;i < off + n_read;i++)
		{
			b[i] = rollByte( (byte)(b[i] ^ (byte)0xd2) , 3 );
		}
		return n_read;
	}
	
	public static byte rollByte(byte src,int bits_to_roll)
	{
		// bits_to_roll : 正なら右に、負なら左に、その絶対値の分だけロールする。
		byte result = src;
		int abs = (bits_to_roll > 0 ? bits_to_roll : -1 * bits_to_roll);
		boolean to_left = (bits_to_roll > 0 ? false : true);
		for (int i = 0;i < abs;i++)
		{
			result = (to_left ? rollByteToLeft(result) : rollByteToRight(result));
		}
		return result;
	}
	public static byte rollByteToLeft(byte src)
	{
		// 左に1ビット回転します。
		byte result = src;
		int msb = result & 0x80; // 最上位ビットを保存
		
		result <<= 1;
		if (msb == 1)
			result |= 0x01;
		else
			result &= 0xfe;
		
		return result;
	}
	public static byte rollByteToRight(byte src)
	{
		// 右に1ビット回転します。
		byte result = src;
		int lsb = result & 0x01; // 最下位ビットを保存
		
		result >>= 1;
		if (lsb == 1)
			result |= 0x80;
		else
			result &= 0x7f;
		
		return result;
	}
}
