/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawkput

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya;
import aya.*;
import aya.node.*;
import aya.eval.*;
import java.io.*;
import java.util.*;

/*
	コメントはC++と同じ。
	
	構文は次の通り。
	
	辞書 := 関数*
	
	関数 := 関数名 ( ':' 関数オプション ) ブロック
	関数名 := + - * / = : ! ; { } % & # " ( ) [ ] < > , ? を含まない文字列
	関数オプション := <文字列>
	
	ブロック := '{' (文 ';'?)* '}'
	文 := 代入文 | 式 | ブロック | '--' | if文 | case文 | switch文 | while文 | for文 | 'break' | 'continue' | 'return'
	
	式 := 項 (符号 項)*
	項 := 符号? 因子 (因子結合子 符号? 因子)*
	因子 := '(' 式 ')' | <数値定数> | <文字列定数> | 関数呼出し | 変数
	符号 := '+' | '-'
	因子結合子 := '*' | '/' | '%'
	関数呼出し := 関数名 引数群?                   このような定義により、引数群の無い未定義関数の呼出しは
	引数群 := '(' (引数? | 引数 (',' 引数)*) ')'   添字の無い変数の参照にリダイレクトされる。
	引数 := 式 | '&' 変数
	変数 := 変数名 ('[' 式 ']')?
	変数名 := + - * / = : ! ; { } % & # " ( ) [ ] < > , ? を含まない文字列
	
	代入文 := 変数 インクリメント演算子 | 変数 代入演算子 式
	インクリメント演算子 := '++' | '--'
	代入演算子 := '='  | '+='  | '-='  | '*='  | '/='  | '%=' |
				  ':=' | '+:=' | '-:=' | '*:=' | '/:=' | '%:='
	
	if文 := 'if' 条件式 ブロック (elseif 条件式 ブロック)* (else ブロック)?
	条件式 := 部分条件式 (条件結合子 部分条件式)*
	部分条件式 := '(' 条件式 ')' | 式 条件比較子 式
	条件比較子 := '==' | '!=' | '<' | '<=' | '>' | '>=' | '_in_' | '!_in_'
	条件結合子 := '||' | '&&'
	
	case文 := 'case' 式 '{' case候補+ '}'
	case候補 := 'when' case条件 ブロック | 'others' ブロック
	case条件 := case部分条件 (',' case部分条件)*
	case部分条件 := <数値定数> | <数値定数> '-' <数値定数> | <文字列定数> | <文字列定数> '-' <文字列定数>
	
	switch文 := 'switch' 式 ブロック
	
	while文 := 'while' 条件式 ブロック
	
	for文 := 'for' 代入文? ';' 条件式? ';' 代入文? ブロック
*/

public class Dictionary
{
	Aya aya;
	Hashtable functions; // {関数名 => Function}
	
	public Dictionary(Aya aya)
	{
		this.aya = aya;
		this.functions = new Hashtable();
	}
	
	public void loadFile(File dicfile)
	{
		LexicalAnalyzer lex = null;
		try
		{
			BufferedReader br;
			if (dicfile.getName().toLowerCase().endsWith(".ayc")) // 暗号化されている。
				br = new BufferedReader(new InputStreamReader(new DataDecodingInputStream(new FileInputStream(dicfile)),Aya.CHARSET));
			else
				br = new BufferedReader(new InputStreamReader(new FileInputStream(dicfile),Aya.CHARSET));
			
			String preprocessed = new Preprocessor(br).doIt();
			lex = new LexicalAnalyzer(preprocessed);
			Parser parser = new Parser(aya,lex);
			while (true) {
				try {
					Function f = parser.parseFunction();
					if (f == null) {
						throw new IllegalTokenException(lex.nextToken());
					}
					else {
						functions.put(f.getName(),f);
					}
				}
				catch (NoMoreTokensException e) {
					break;
				}
			}
		}
		catch (IllegalTokenException e) {
		    System.err.println(dicfile.getName() + ":"
				       + e.getToken().getPosition().getRow()
				       + ": Syntax error occured near \""
				       + e.getToken().getToken() + "\"");
		}
		catch (Exception e) {
			System.err.println("Couldn't load file "+dicfile.getAbsolutePath());
			System.err.println("lex: "+lex);
			e.printStackTrace();
		}
	}
	
	public Function getFunction(String name) {
		// システムコールよりユーザー定義関数の方を優先する。暫定。
		
		Function f = (Function)functions.get(name);
		if (f != null) {
			return f;
		}
		
		SystemCall syscall = aya.getSystemCall().get(name);
		if (syscall != null) {
			return syscall;
		}
		
		return null;
	}
	
	public void undefFunction(String name) {
		functions.remove(name);
	}
}
