/*
    $Id: Dictionary.java,v 1.6 2004/01/29 12:14:45 phonohawk Exp $
    
    "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
    Copyright (C) 2001 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package satori;
import satori.*;
import java.io.*;
import java.util.*;

public class Dictionary
{
    Hashtable sentences; // key : (String)̖OB΋󔒕B , value : LinedDataBlock    
    Hashtable words; // key : (String)PꃊXg̖OB , value : LinedDataBlock
    // content Vector ; content : (String)̒PB
	
    Vector keywords_to_be_anchored; // <String>AJ[L[[h
    
    Satori satori;
    
    public Dictionary(Satori satori) {
	this.satori = satori;
	sentences = new Hashtable();
	words = new Hashtable();
	keywords_to_be_anchored = new Vector();
    }
    
    public void loadAllDictionariesInDirectory(File dir) {
	if (!dir.exists()) return;
	if (!dir.isDirectory()) return;
	
	File[] items = dir.listFiles();
	final int n_items = items.length;
	for (int i = 0;i < n_items;i++) {
	    String fname = items[i].getName();
	    if ((fname.startsWith("dic") &&
		 (fname.endsWith(".txt") || fname.endsWith(".sat")) &&
		 !fname.equals("satori_savedata.txt")) || // Z[uf[^JԂǂł܂ƂhB
		fname.equals("satori_conf.txt")) { // @SAORÏׂɓǂށB
		loadDicFile(items[i]);
	    }
	}
    }
    
    public void resetAll() {
	// f[^ɂB
	sentences.clear();
	words.clear();
    }
    
    public void loadDicFile(File f) {
	// dic[id].txtdic[id].sat[h܂B
	boolean auto_anchor = false;
	if (f.getName().startsWith("dicAnchor")) {
	    auto_anchor = true;
	}
	
	try {
	    InputStream is;
	    if (f.getName().endsWith(".sat")) {
		is = makeInputStreamOfDecodingSat(new FileInputStream(f));
	    }
	    else {
		is = new FileInputStream(f);
	    }
	    BufferedReader br = new BufferedReader(new InputStreamReader(is,"SJIS"));
	    
	    StringBuffer buf = new StringBuffer();
	    boolean in_comment_block = true;
	    while (true) {
		String line = br.readLine();
		if (line == null) {
		    // eofBobt@tbVďIB
		    evaluateDictBlock(buf.toString().trim(),auto_anchor); // Ō̉s͍B
		    break;
		}
		line = satori.getPreTranslator().translate(line);
		line = Satori.encodeEscapedChars(line); // GR[h
		line = ZenHanTranslator.convZenToHan(line); // SpL/l𔼊pɂB
		
		if (line.length() == 0) {
		    buf.append('\n');
		    continue;
		}
		else if (line.charAt(0) == '#' || (in_comment_block && line.charAt(0) != '*' && line.charAt(0) != '@')) {
		    continue; // comment
		}
		else if (line.startsWith('*' + Satori.ZENKAKU_COMMENT)) {
		    // RgubNJnB
		    in_comment_block = true;
		    continue;
		}
		else if (line.charAt(0) == '*' || line.charAt(0) == '@') {
		    // Ŏn܂sRgubNtONAăobt@tbVĎ̃ubNցB
		    // t@C擪̃ubNobt@ɂ͉ĂȂB
		    in_comment_block = false;
		    
		    String validcontent = buf.toString().trim();
		    if (validcontent.length() > 0) {
			evaluateDictBlock(validcontent,auto_anchor);
		    }
		    buf.setLength(0);
		}
		buf.append(line+'\n');
	    }
	    
	    br.close();
	}
	catch (Exception e) {
	    System.out.println("Dictionary error : Couldn\'t load dic file "+f.getName()+".");
	}
    }
    
    private InputStream makeInputStreamOfDecodingSat(InputStream is) throws Exception {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	while (true) {
	    byte[] line = readUntilCRLF(is);
	    if (line == null) break; // EOF
	    
	    byte[] decoded = decode(line);
	    decoded = decode(decoded);
	    
	    baos.write(decoded);
	    baos.write((int)0x0d); // CR
	    baos.write((int)0x0a); // LF
	}
	return new ByteArrayInputStream(baos.toByteArray());
    }
    private byte[] readUntilCRLF(InputStream is) throws Exception {
	long count = 0;
	
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	while (true) {
	    int c = is.read();
	    if (c == -1) { // EOF
		if (count == 0) return null; // ܂ꕶǂłȂ̂EOFɒB̂łŏIB
		break;
	    }
	    else if (c == 0x0d) { // CR
		is.read(); // LFǂݔ΂B
		break;
	    }
	    else {
		baos.write(c);
		count++;
	    }
	}
	return baos.toByteArray();
    }
    private byte[] decode(byte[] src) {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	for (int i = 0;i < src.length;i += 2) {
	    baos.write((int)src[i]);
	}
	for (int i = src.length - ((src.length & 0x01) == 1 ? 2 : 1) ; i >= 0 ; i -= 2) {
	    baos.write((int)src[i]);
	}
	return baos.toByteArray();
    }
    
    public static String insertReturnAtColons(String src) {
	// ܂ꂽR̒OCRLF}B
	StringBuffer buf = new StringBuffer(src);
	int pos = 0;
	while (pos < buf.length()) {
	    if (buf.charAt(pos) != ':') {
		pos++;
		continue;
	    }
	    else if (pos == 0 || pos > 0 && buf.charAt(pos-1) == '\n') {
		pos++;
		continue;
	    }
	    else {
		buf.insert(pos,"\r\n");
	    }
	}
	return buf.toString();
    }
    
    public void evaluateDictBlock(String block,boolean auto_anchor) {
	if (block.length() == 0) return;
	
	try {
	    BufferedReader br = new BufferedReader(new StringReader(block));
	    
	    // 擪̍s̕ŃubN̎ނ𔻕ʁB
	    String header = br.readLine();
	    while (header.length() > 0 && header.charAt(header.length()-1) == Satori.ZENKAKU_FAI) {
		// ӂŏIĂ玟̍sƌqB
		header = header.substring(0,header.length()-1) + br.readLine();
	    }
	    
	    if (header.charAt(0) == '*') {
		// AX^XNF
		String sentence_name = (header.length() == 1 ? "" : header.substring(1));
		StringBuffer sentence_content = new StringBuffer();
		while (true) {
		    String line = br.readLine();
		    if (line == null) break; // I
		    
		    sentence_content.append(insertReturnAtColons(line)+'\n');
		}
		if (sentence_content.length() > 0) { // ̃ZeXȂ΂B
		    sentence_content.setLength(sentence_content.length()-1); // Ōɂ]ȉsB
		}
		
		// ZeXubNB
		LinedDataBlock sentenceblocks = (LinedDataBlock)sentences.get(sentence_name);
		if (sentenceblocks == null) { // ߂ČꂽZeXB
		    sentenceblocks = new LinedDataBlock(satori.getVariable(), sentence_name, LinedDataBlock.TYPE_SENTENCE);
		    sentences.put(sentence_name, sentenceblocks);
		}
		sentenceblocks.add(sentence_content.toString()); // o^
		
		// auto_anchortrueȂkeywords_to_be_anchoredɃZeXǉB
		if (auto_anchor) keywords_to_be_anchored.add(sentence_name);
	    }
	    else if (header.charAt(0) == '@') {
		// Abg}[NFPꃊXg
		String wordlist_name = header.substring(1);
		while (true) {
		    String word = br.readLine();
		    if (word == null) break; // end
		    
		    LinedDataBlock wordlist = (LinedDataBlock)words.get(wordlist_name);
		    if (wordlist == null) {
			wordlist = new LinedDataBlock(satori.getVariable(), wordlist_name, LinedDataBlock.TYPE_WORD);
			words.put(wordlist_name,wordlist);
		    }
		    wordlist.add(word); // o^
		}
	    }
	    
	    br.close();
	}
	catch (Exception e) {
	    System.out.println("Dictionary error : evaluateDictBlock caused an exception.");
	    System.out.println(e.toString());
	}
    }
    
    public String getSentenceMatchesTo(String str) {
	// strɍłvZeXubNԂ܂B
	// vȂꍇnullԂ܂B
	Vector vec_points,vec_sentencenames;
	vec_points = new Vector();
	vec_sentencenames = new Vector();
	for (Enumeration e = sentences.keys();e.hasMoreElements();) {
	    String sentence_name = (String)e.nextElement();
	    if (sentence_name.length() == 0) continue; // ̃ZeX
	    
	    // Pꖈɕ
	    StringTokenizer st_words = new StringTokenizer(sentence_name,"\u3000\u0020");
	    int points = 0; // v
	    while (st_words.hasMoreTokens()) {
		String word = st_words.nextToken();
		
		if (str.indexOf(word) != -1) { // vB
		    points += 10 + countBytesOfString(word) / 4;
		}
		else {
		    points--;
		}
	    }
	    
	    // 1_ȏȂ_ƃZeXo^
	    if (points >= 1) {
		vec_points.add(new Integer(points));
		vec_sentencenames.add(sentence_name);
	    }
	}
	if (vec_points.size() == 0) return null; // vȂB
	
	// vec_points̍ől߂B
	int max = 0;
	final int n_items = vec_points.size();
	for (int i = 0;i < n_items;i++) {
	    int pt = ((Integer)vec_points.elementAt(i)).intValue();
	    if (max < pt) max = pt;
	}
	
	// ̍őlZeXW߂B
	Vector maxlevel_sentence_names = new Vector();
	for (int i = 0;i < n_items;i++) {
	    int pt = ((Integer)vec_points.elementAt(i)).intValue();
	    if (pt == max) {
		maxlevel_sentence_names.add(vec_sentencenames.elementAt(i));
	    }
	}
	
	// ZeXIԁB
	int sel_sname = (int)((Math.random() * Integer.MAX_VALUE) % maxlevel_sentence_names.size());
	String selected_sentence_name = (String)maxlevel_sentence_names.elementAt(sel_sname);
	
	// ZeXIԁB
	LinedDataBlock maxlevel_sentences = getSentenceBlock(selected_sentence_name);
	return maxlevel_sentences.get();
    }
    
    public static int countBytesOfString(String str)
    {
	// 0x00000000 ` 0x000000ff = 1bytes
	// 0x00000100 ` 0x0000ffff = 2bytes
	// 0x00010000 ` 0x00ffffff = 3bytes
	// 0x01000000 ` 0xffffffff = 4bytes
	int result = 0;
	
	int len = str.length();
	for (int i = 0;i < len;i++) {
	    char c = str.charAt(i);
	    
	    if (c <= 0x000000ff) result++;
	    else if (c <= 0x0000ffff) result += 2;
	    else if (c <= 0x00ffffff) result += 3;
	    else if (c <= 0xffffffff) result += 4;
	}
	
	return result;
    }
	
    public String autoAnchor(String src) {
	// src̎AJ[L[[hAJ[B
	StringBuffer buf = new StringBuffer(src);
	int pos = 0;
	while (pos < buf.length()) {
	    boolean found = false;
	    
	    final int n_keywords = keywords_to_be_anchored.size();
	    for (int i = 0;i < n_keywords;i++) {
		String keyword = (String)keywords_to_be_anchored.elementAt(i);
		if (buf.toString().startsWith(keyword,pos)) {
		    // L[[hnP[
		    String replaced_after = "\\_a["+keyword+']'+keyword+"\\_a";
		    buf.replace(pos,pos+keyword.length(),replaced_after);
		    pos += replaced_after.length();
		    found = true;
		    break;
		}
	    }
	    
	    if (!found) pos++;
	}
	return buf.toString();
    }

    public LinedDataBlock getSentenceBlock(String name) {
	return getSentenceBlock(name, false);
    }
    public LinedDataBlock getSentenceBlock(String name, boolean empty_if_not_exists) {
	// w肳ꂽOZeXubN̔zԂ܂B
	// _g[N̏ꍇ͋󔒕w肵ĉB
	LinedDataBlock result = (LinedDataBlock)sentences.get(name);
	if (result == null && empty_if_not_exists) {
	    return new LinedDataBlock(satori.getVariable(), name, LinedDataBlock.TYPE_SENTENCE);
	}
	else {
	    return result;
	}
    }
    
    public LinedDataBlock getWordList(String name) {
	return getWordList(name, false);
    }
    public LinedDataBlock getWordList(String name, boolean empty_if_not_exists) {
	// w肳ꂽOP̔zԂ܂B
	LinedDataBlock result = (LinedDataBlock)words.get(name);
	if (result == null && empty_if_not_exists) {
	    return new LinedDataBlock(satori.getVariable(), name, LinedDataBlock.TYPE_WORD);
	}
	else {
	    return result;
	}
    }

    public void updateSettingOfAllBlocks() {
	for (Iterator ite = sentences.values().iterator(); ite.hasNext(); ) {
	    LinedDataBlock block = (LinedDataBlock)ite.next();
	    block.updateSetting();
	}
	for (Iterator ite = words.values().iterator(); ite.hasNext(); ) {
	    LinedDataBlock block = (LinedDataBlock)ite.next();
	    block.updateSetting();
	}
    }
}
